{% extends "layout.html" %}
{% block content %}
<div class="row">
  <div class="content" id="bookmark_view">
	<h2></h2>
	<h4><a></a></h4>
	<p id="desc"></p>
	<div id="category"><a class="btn btn-primary"></a></div>
	<div id="tags"></div>
  </div>
</div>
{% endblock %}
